#################################################
# This script queries FIADB (FS_NIMS_FIADB_RMRS and FS_NIMS_RMRS) for data to be used to estimate
# LAI using multiple methods (in another script). It queries plot, cond, tree, and veg data, and
# writes output CSV files for each canopy stratum (overstory and understory).
# Subsequent scripts estimate plot-level LAI based on these CSV files.
#
# LAI estimation methods require FIA data in different formats.
# 1) Two methods require using plot and cond data to aggregate tree canopy cover to the plot-level.
# 2) One method requires tree data from live trees (species, height, diameter).
# 3) One method requires overstory and understory canopy volume estimates. Overstory volume is 
# based on tree data (height and compacted crown ratio of live trees) combined with cond-level 
# tree canopy cover; then cond-level volume is aggregated to plot level. Understory volume is 
# based on veg cover by life form and height class, aggregated to cond and then plot level.
# 4) One method requires percent cover by field forest type and veg life form.
#################################################
# 	To run this script, the user will first need to download the requisite tables (PLOT, COND, TREE,
# 		P2VEG_SUBP_STRUCTURE) from the public FIA database, FIADB, using one of two methods:
#	1) CSV files for the appropriate U.S. state(s) from the FIA DataMart 	
#		(https://apps.fs.usda.gov/fia/datamart/datamart.html). If this method is used, the CSV files 
#		must be imported (read) into R before proceeding.
#	2) Use r package 'rFIA' to download the tables from FIADB for the U.S. state(s) of interest.
#
# 	These tables must exist in the R workspace with all lowercase names (e.g., "plot") before running this script.
#################################################

### User inputs:

	# Location of output datasets from previous scripts:
	
		input_path <- "E:/user/outfiles/"	
		output_path <- "E:/user/outfiles/"		
		
#################################################
### PLOT DATA
#################################################
# filter to include only necessary variables, for efficiency:
plot_ann <- subset(plot[,c("CN","STATECD","LAT","LON","PLOT_STATUS_CD","MEASYEAR","INVYR","CYCLE","SUBCYCLE")])		

# must constrain the analysis to plots where PLOT_STATUS_CD is 1 or 2:
plot_ann <- subset(plot_ann, PLOT_STATUS_CD<=2)	
# Add constraints on specific years, if desired, using INVYR or MEASYEAR. See Burrill et al. (2018) for documentation of these variables.

dim(plot_ann)
table(plot_ann$STATECD,plot_ann$INVYR)
# convert PLT_CN to character (from numeric) and delete the numeric version
plot_ann$PLT_CN <- as.character(plot_ann$CN)
# get rid of original CN
plot_ann <- subset(plot_ann, select=-c(CN)); str(plot_ann)

### WRITE OUTPUT CSV
write.csv(plot_ann, paste(input_path,"plot_ann.csv", row.names = FALSE)
#################################################


#################################################
### COND DATA
#################################################
### used by: lai4, lai5

# filter to include only necessary variables, for efficiency:
cond_ann <- subset(cond[,c("PLT_CN","CN","CONDID","CONDPROP_UNADJ","LIVE_CANOPY_CVR_PCT","COND_STATUS_CD","FORTYPCD"))		

# must constrain the analysis to plots where PLOT_STATUS_CD is 1 or 2:
cond_ann <- subset(cond_ann, PLOT_STATUS_CD<=2)
# Add constraints on specific years, if desired, using INVYR or MEASYEAR. See Burrill et al. (2018) for documentation of these variables.

dim(cond_ann)
# convert PLT_CN to character (from numeric) and delete the numeric version
cond_ann$PLT_CN2 <- as.character(cond_ann$PLT_CN); cond_ann$PLT_CN <- cond_ann$PLT_CN2
cond_ann <- subset(cond_ann, select=-c(PLT_CN2))
cond_ann$COND_CN <- as.character(cond_ann$CN)
cond_ann <- subset(cond_ann, select=-c(CN)); str(cond_ann)
# verify that COND_STATUS_CD>1 for conditions where canopy cover is NA
# table output should show no value==1 (plot_status_cd==1 recoded to 0)
temp <- ifelse(is.na(cond_ann$LIVE_CANOPY_CVR_PCT)==T, cond_ann$COND_STATUS_CD, 0)
table(temp); summary(cond_ann$LIVE_CANOPY_CVR_PCT) 
# recode LCC for nonforest, census water, and noncensus water <- 0
cond_ann$LIVE_CANOPY_CVR_PCT <- ifelse(cond_ann$COND_STATUS_CD!=1, 
	ifelse(cond_ann$COND_STATUS_CD!=5, 0, cond_ann$LIVE_CANOPY_CVR_PCT), 
	ifelse(is.na(cond_ann$LIVE_CANOPY_CVR_PCT)==T, cond_ann$LIVE_CANOPY_CVR_PCT,
	cond_ann$LIVE_CANOPY_CVR_PCT))
summary(cond_ann$LIVE_CANOPY_CVR_PCT) # note the number of NA values
# flag sampled vs. nonsampled plots
cond_ann$sampled <- ifelse(cond_ann$COND_STATUS_CD<5, 1, 0)
cond_ann$condprop_sampl <- ifelse(cond_ann$sampled==1, cond_ann$CONDPROP_UNADJ, 0)
table(cond_ann$sampled,cond_ann$COND_STATUS_CD)
cond_ann[is.na(cond_ann)] <- 0	# convert NA values to 0

### WRITE OUTPUT CSV - un-aggregated cond data; use for lai4 and lai5
write.csv(cond_ann, paste(input_path,"cond_ann.csv", row.names = FALSE)	
#################################################


#################################################
### COND, AGGREGATED TO PLOT LEVEL ###
#################################################
### USED BY: lai1, lai2 (note that cond-level data are used for lai4 and lai5, and aggregated later)

# adjust by condprop_sampl (condprop_unadj for sampled conditions)
cond_ann$lcc_cond_adj <- cond_ann$LIVE_CANOPY_CVR_PCT*cond_ann$condprop_sampl 
# aggregate to plot level
plot_cc <- aggregate(cond_ann[,c("lcc_cond_adj","condprop_sampl","CONDPROP_UNADJ")],
	by=list(PLT_CN=cond_ann$PLT_CN), FUN=sum, na.rm=T)
plot_cc$lcc_plot <- plot_cc$lcc_cond_adj/plot_cc$condprop_sampl
summary(plot_cc$lcc_plot)
# CHECK : maximum value of lcc_plot should be 100
# CHECK : aggregated values of condprop_sampl should include values<=1; condprop_unadj should all = 1
# THEN subset the data.frame to contain only PLT_CN, condprop_sampl, lcc_plot
plot_cc <- subset(plot_cc[,c("PLT_CN","condprop_sampl","lcc_plot")])
### add PLOT_STATUS_CD to the aggregated condition data at the plot level
plot_status <- subset(plot_ann[,c("PLT_CN","PLOT_STATUS_CD","lat","lon")])
plot_cc <- merge(x=plot_status,y=plot_cc,by="PLT_CN")

### WRITE OUTPUT CSV - cond-level cc aggregated to plot level; use for lai1, lai2
write.csv(plot_cc, paste(input_path,"plot_cc.csv", row.names = FALSE) 
#################################################


#################################################
### TREE DATA
#################################################
### used by: lai3
# Variables needed: DIA, SPCD; constraints: live trees only
trees <- subset(tree[,c("PLT_CN","CONDID","STATUSCD","DIA","SPCD")])
trees <- subset(trees, STATUSCD==1)
dim(trees)
# convert PLT_CN to character (from numeric) and delete the numeric version
trees$PLT_CN2 <- as.character(trees$PLT_CN); trees$PLT_CN <- trees$PLT_CN2
trees <- subset(trees, select=-c(PLT_CN2)); str(trees)
trees <- subset(trees, is.na(DIA)==F)

### WRITE OUTPUT CSV - tree-level spcd and dia; use for lai3
write.csv(trees, paste(input_path,"trees.csv", row.names = FALSE) 
#################################################


#################################################
### UNDERSTORY VEG DATA
#################################################
### used by: lai4, lai5
# Variables needed: PLT_CN, LAYER, COVER_PCT, GROWTH_HABIT_CD, CONDID, CONDPROP_UNADJ
# Get plot-level overstory canopy cover from LAI dataset.

understory <- subset(P2VEG_SUBP_STRUCTURE[,c("PLT_CN","CONDID","SUBP","GROWTH_HABIT_CD","LAYER","COVER_PCT")])

# must merge with COND data to get CONDPROP_UNADJ
understory <- merge(understory, cond_ann, by=list("PLT_CN","CONDID"))

dim(understory)
# convert PLT_CN to character (from numeric) and delete the numeric version
understory$PLT_CN2 <- as.character(understory$PLT_CN); understory$PLT_CN <- understory$PLT_CN2
understory <- subset(understory, select=-c(PLT_CN2)); str(understory)

### WRITE OUTPUT CSV - unprocessed veg data for estimating understory LAI
write.csv(understory, paste(input_path,"understory.csv", row.names = FALSE) 
#################################################
