#################################################
# This script uses outputs from previous queries of FIADB (FIADB_queries_public.R) to estimate plot-level LAI 
# 	using multiple methods. 
# It reads CSVs of plot, cond, tree, and veg data, and applies four LAI estimation methods to get
# 	plot-level LAI, and then writes output files.
#
# LAI estimation methods require FIA data of different types and formats.
# Two methods (lai1 and lai2) require using plot and cond data to aggregate tree canopy cover 
# 	to the plot-level.
# One method (lai4) requires percent cover by field forest type and veg life form, plus a 
# 	lookup table for clumping indices by forest type and life form.
# LAI5 method is no longer used; it involved intersecting plot-level proportions of understory and overstory
# 	percent cover with MODIS LAI from a specific date.
#################################################

### User inputs:

	# Location of output datasets from previous scripts:
	
		input_path <- "E:/user/outfiles/"	
		output_path <- "E:/user/outfiles/"	
		
#################################################
##### READ INPUT DATA FILES (from FIADB_queries.R
#################################################
# for lai1 and lai2:
plot_cc <- read.csv(paste(input_path,"plot_cc.csv", header = T, na.strings = ".", stringsAsFactors=F) # make "." NA during import
# This step occurs for all LAI estimation methods; its purpose is to convert PLT_CN to character (which is how it is defined within the public FIADB). Otherwise R will convert it to numeric.
plot_cc$PLT_CNTMP <- as.character(plot_cc$PLT_CN); plot_cc$PLT_CN <- plot_cc$PLT_CNTMP
plot_cc <- subset(plot_cc, select=-c(PLT_CNTMP)); str(plot_cc)

# for lai3:
trees <- read.csv(paste(input_path,"trees.csv", header = T, na.strings = ".", , stringsAsFactors=F) # make "." NA during import
trees$PLT_CNTMP <- as.character(trees$PLT_CN); trees$PLT_CN <- trees$PLT_CNTMP
trees <- subset(trees, select=-c(PLT_CNTMP)); str(trees)

# for lai4:
understory <- read.csv(paste(input_path,"understory.csv", header = T, na.strings = ".", stringsAsFactors=F) # make "." NA during import
understory$PLT_CNTMP <- as.character(understory$PLT_CN); understory$PLT_CN <- understory$PLT_CNTMP
understory <- subset(understory, select=-c(X,PLT_CNTMP)); str(understory)

cond_ann <- read.csv(paste(input_path,"cond_ann.csv", header = T, na.strings = ".", stringsAsFactors=F) # make "." NA during import
cond_ann$PLT_CNTMP <- as.character(cond_ann$PLT_CN); cond_ann$PLT_CN <- cond_ann$PLT_CNTMP
cond_ann <- subset(cond_ann, select=-c(PLT_CNTMP)); str(cond_ann)
cond_ann$COND_CNTMP <- as.character(cond_ann$COND_CN); cond_ann$COND_CN <- cond_ann$COND_CNTMP
cond_ann <- subset(cond_ann, select=-c(COND_CNTMP)); str(cond_ann)

#################################################
##### METHOD 1: SCALE LAI WITH CANOPY COVER #####
#################################################
# calculate plot-level overstory LAI:
lai_max <- 5.3 	# LAI max from plots in western MT (Pierce & Running 1988)
#lai_max <- 20.8 # max LAI from NDVI in Glacier NP (White et al. 1997)
plot_lai1 <- plot_cc
plot_lai1$lai1 <- plot_lai1$lcc_plot * (lai_max/100)
#hist(plot_lai1$lai1[plot_lai1$lai1!=0])
#summary(plot_lai1$lai1[plot_lai1$lai1!=0])
write.csv(plot_lai1, paste(output_path,"plot_lai1.csv", row.names = FALSE)


#################################################
##### METHOD 2: POMEROY (2002) EQ. FROM VARHOLA ET AL. (2010) #####
#################################################
plot_lai2 <- plot_cc
plot_lai2$lai2 <- ifelse(plot_lai2$lcc_plot>0, exp((plot_lai2$lcc_plot/100 - 0.55)/0.29), 0)
#hist(plot_lai2$lai2[plot_lai2$lai2!=0])
#summary(plot_lai2$lai2[plot_lai2$lai2!=0])
write.csv(plot_lai2, paste(output_path,"plot_lai2.csv", row.names = FALSE)


#################################################
##### METHOD 3: ALLOMETRIC MODELS OF KAUFMANN ET AL. (1982; table 2) #####
#################################################
# steps: use DBH^2 (DBH^4 for aspen) and a species-specific coefficient to get effective project leaf area (EPLA in Table 2 of Kaufmann); then divide by the plot area (in ft^2) to get plot LAI.
trees$dia_cm <- trees$DIA*2.54
coeff_spruce <- 0.0752
coeff_fir <- 0.0681
coeff_pine <- 0.0261
coeff1_aspen <- 0.0420
coeff2_aspen <- 0.0000155  # coeff <- 0.132 (*dbh^2) and 0.0000486 (*dbh^4) for aspen
# calculate tree-level LAI:
trees$lai <- ifelse(trees$SPCD<=69, coeff_fir*trees$dia_cm^2, 
	ifelse((trees$SPCD>=90 & trees$SPCD<=99)|(trees$SPCD>=200 & trees$SPCD<=299), coeff_spruce*trees$dia_cm^2,
	ifelse((trees$SPCD>=70 & trees$SPCD<=89)|trees$SPCD>=100, coeff_pine*trees$dia_cm^2,
	ifelse(trees$SPCD>=300, coeff1_aspen*trees$dia_cm^2 - coeff2_aspen*trees$dia_cm^4, 0))))
# aggregate to plot-level LAI; note that plot area = 7,238.22336 ft^2 = 672.453 m^2
plot_lai3 <- setNames(aggregate(trees$lai, by=list(PLT_CN=trees$PLT_CN), FUN=sum, na.rm=T),c("PLT_CN","lai3"))
plot_lai3$lai3 <- plot_lai3$lai3 / 672.453  	# divide by conversion factor to get area of 4 subplots in m2
plot_lai3 <- merge(plot_lai3, plot_cc)
plot_lai3$lai3 <- ifelse(is.na(plot_lai3$lai3)==T, 0, plot_lai3$lai3)
#hist(plot_lai3$lai3[plot_lai3$lai3!=0])
#summary(plot_lai3$lai3[plot_lai3$lai3!=0])
write.csv(plot_lai3, paste(output_path,"plot_lai3.csv", row.names = FALSE)


#################################################
##### METHOD 4: APPLY CLUMPING INDICES OF CHEN ET AL. (2005) TO OVERSTORY VS. UNDERSTORY (LAYER 5) COVER #####
#################################################
# EQUATION: See Eq. 2 in Breda et al. (2003), where P(theta) = gap fraction (i.e., LCC),
# theta = 0 (because gap fraction is based on aerial view), and G(theta) equals the clumping index.
# Clumping indices are assigned below based on crosswalk of Chen et al.'s (2005) Table 3 to FIA
# Growth Habit Codes and Forest Types.

### UNDERSTORY ###
### Use only Layer 5 (exclude indivdiual layers and exclude tally trees; LCC includes seeds and saps)
under_subp <- subset(understory, LAYER==5 & GROWTH_HABIT_CD!="TT")

# aggregate COVER_PCT from subplot to condition, by GROWTH_HABIT_CD
cond_lai4_under <- aggregate(under_subp[,c("COVER_PCT")], 
	by=list(PLT_CN=under_subp$PLT_CN,CONDID=under_subp$CONDID, CONDPROP_UNADJ=under_subp$CONDPROP_UNADJ,GROWTH_HABIT_CD=under_subp$GROWTH_HABIT_CD), FUN=mean, na.rm=T)
# rename aggregated column
cond_lai4_under$COVER_PCT <- cond_lai4_under$x; cond_lai4_under <- subset(cond_lai4_under, select=-c(x))

# aggregate COVER_PCT from condition to plot, by GROWTH_HABIT_CD
plot_lai4_under_grfm <- aggregate(cond_lai4_under[,c("COVER_PCT")], 
	by=list(PLT_CN=cond_lai4_under$PLT_CN, GROWTH_HABIT_CD=cond_lai4_under$GROWTH_HABIT_CD), FUN=mean, na.rm=F)
# rename aggregated column
plot_lai4_under_grfm$cover_pct <- plot_lai4_under_grfm$x
plot_lai4_under_grfm <- subset(plot_lai4_under_grfm, select=-c(x))

# aggregate COVER_PCT among all growth habits:
plot_lai4_under <- aggregate(plot_lai4_under_grfm[,c("cover_pct")], 
	by=list(PLT_CN=plot_lai4_under_grfm$PLT_CN), FUN=sum, na.rm=F)
plot_lai4_under$cover_pct <- plot_lai4_under$x; plot_lai4_under <- subset(plot_lai4_under, select=-c(x))

# Clumping indices for the understory: 0.75 ("sparse herbaceous or sparse shrub cover") 
under_clumpindex <- 0.75

# Substitute cover = 99 where it is recorded as 100 to avoid LAI=inf:
plot_lai4_under$cover_pct <- ifelse(plot_lai4_under$cover_pct>99, 99, plot_lai4_under$cover_pct)

# LAI EQUATION (note log = ln in R):
plot_lai4_under$lai4_under <- log(1-plot_lai4_under$cover_pct/100) / (-0.5*under_clumpindex)
#plot(plot_lai4_under$cover_pct,plot_lai4_under$lai4_under)

### OVERSTORY ###
### Based on FORTYPCD
cond_lai4_over <- cond_ann
# Clumping indices for the overstory 
cond_lai4_over$clump_ind <- ifelse(cond_lai4_over$COND_STATUS_CD==5, 0.72, # nonsampled land (mosaic class)
	ifelse(cond_lai4_over$COND_STATUS_CD>=2 & cond_lai4_over$COND_STATUS_CD<5, 0.75, # nonforest land or census/noncensus/census water
	ifelse(cond_lai4_over$FORTYPCD==999, 0.72, 	# nonstocked
	ifelse(cond_lai4_over$FORTYPCD<320, 0.62,	# tree cover, needleleaf, evergreen
	ifelse(cond_lai4_over$FORTYPCD<=321, 0.68,	# tree cover, needleleaf, deciduous
	ifelse(cond_lai4_over$FORTYPCD<400, 0.62,	# tree cover, needleleaf, evergreen
	ifelse(cond_lai4_over$FORTYPCD<410, 0.69,	# tree cover, mixed leaf type (pine/oak mixes)
	ifelse(cond_lai4_over$FORTYPCD<999 & cond_lai4_over$LIVE_CANOPY_CVR_PCT<65, 0.70,	# tree cover, broadleaf, deciduous, open
	ifelse(cond_lai4_over$FORTYPCD<999 & cond_lai4_over$LIVE_CANOPY_CVR_PCT>=65, 0.69,	# tree cover, broadleaf, deciduous, closed
	0)))))))))

# Substitute canopy cover = 99 where it is recorded as 100 to avoid LAI=inf:
cond_lai4_over$LIVE_CANOPY_CVR_PCT <- ifelse(cond_lai4_over$LIVE_CANOPY_CVR_PCT==100, 99, cond_lai4_over$LIVE_CANOPY_CVR_PCT)

# LAI EQUATION (note log = ln in R):
cond_lai4_over$lai4_over <-	log(1-cond_lai4_over$LIVE_CANOPY_CVR_PCT/100) / (-0.5*cond_lai4_over$clump_ind)
#plot(cond_lai4_over$LIVE_CANOPY_CVR_PCT,cond_lai4_over$lai4_over)

# multiply by condprop_unadj to weight LAI
cond_lai4_over$lai4_over_wtd <- cond_lai4_over$lai4_over*cond_lai4_over$condprop_sampl

# aggregate the area-weighted LAI to plot (use FUN=sum)
plot_lai4_over <- aggregate(cond_lai4_over[,c("lai4_over_wtd")], by=list(PLT_CN=cond_lai4_over$PLT_CN), FUN=mean, na.rm=F)
# rename aggregated column
plot_lai4_over$lai4_over <- plot_lai4_over$x; plot_lai4_over <- subset(plot_lai4_over, select=-c(x))

# merge LAI4 understory and overstory
plot_lai4 <- merge(plot_lai4_over,plot_lai4_under)
write.csv(plot_lai4, paste(output_path,"plot_lai4.csv", row.names = FALSE)


#################################################
##### METHOD 5: PARTITION MODIS/GLAS LAI INTO OVERSTORY AND UNDERSTORY BASED ON MEASURED CANOPY VOLUMES #####
#################################################
# BAIL. JUST BAIL. OK, calculate the proportion of overstory vs. understory at the plot level - but don't interset with MODIS. It's just too messy, heterogeneous, and not useful at this point. But it might be very useful to instead resample the output grids based on spatial modeling of the other methods, and then compare them to MODIS LAI.


#################################################
##### MERGE DATASETS FOR ALL METHODS
#################################################
# compare boxplots of all overstory LAI methods, excluding zeros:
boxplot(plot_lai1$lai1[plot_lai1$lai1!=0], plot_lai2$lai2[plot_lai2$lai2!=0], plot_lai3$lai3[plot_lai3$lai3!=0], plot_lai4$lai4_over[plot_lai4$lai4_over!=0])
# compare boxplots, including zeros:
boxplot(plot_lai1$lai1, plot_lai2$lai2, plot_lai3$lai3, plot_lai4$lai4_over)

plot_lai <- merge(plot_lai1,plot_lai2[,c("PLT_CN","lai2")], by="PLT_CN")
plot_lai <- merge(plot_lai, plot_lai3[,c("PLT_CN","lai3")], by="PLT_CN")
plot_lai <- merge(plot_lai, plot_lai4[,c("PLT_CN","lai4_over","lai4_under")], by="PLT_CN")

# Read plot_ann query output (from FIADB_queries.R) to get MEASYEAR.
plot_ann <- read.csv(paste(output_path,"plot_ann.csv", header = T, na.strings = ".", stringsAsFactors=F) # make "." NA during import
plot_ann$PLT_CNTMP <- as.character(plot_ann$PLT_CN); plot_ann$PLT_CN <- plot_ann$PLT_CNTMP
plot_ann <- subset(plot_ann, select=-c(lat_true, lon_true, PLT_CNTMP)); str(plot_ann)
plot_lai <- subset(plot_lai, select=-c(PLOT_STATUS_CD)); str(plot_lai)

# merge with LAI dataset
plot_lai <- merge(plot_ann, plot_lai, all.x=T)

# change NA values for overstory LAI on nonforest plots to 0
plot_lai$lai1 <- ifelse(plot_lai$PLOT_STATUS_CD==2, 0, plot_lai$lai1)
plot_lai$lai2 <- ifelse(plot_lai$PLOT_STATUS_CD==2, 0, plot_lai$lai2)
plot_lai$lai3 <- ifelse(plot_lai$PLOT_STATUS_CD==2, 0, plot_lai$lai3)
plot_lai$lai4_over <- ifelse(plot_lai$PLOT_STATUS_CD==2, 0, plot_lai$lai4_over)

write.csv(plot_lai, paste(output_path,"plot_lai.csv", row.names = FALSE)

#########
# Read data (this reads the data compiled and written above - it is intended to prepare the user for subsequent analysis):
plot_lai <- read.csv(paste(output_path,"plot_lai.csv", header = T, na.strings = ".", stringsAsFactors=F) # make "." NA during import

plot_lai$condprop_sampl <- as.numeric(plot_lai$condprop_sampl)
plot_lai$lcc_plot <- as.numeric(plot_lai$lcc_plot)
plot_lai$lmcc_plot <- as.numeric(plot_lai$lmcc_plot)
plot_lai$lai1 <- as.numeric(plot_lai$lai1)
plot_lai$lai2 <- as.numeric(plot_lai$lai2)
plot_lai$lai3 <- as.numeric(plot_lai$lai3)
plot_lai$lai4_over <- as.numeric(plot_lai$lai4_over)
plot_lai$lai4_under <- as.numeric(plot_lai$lai4_under)

